@@ -6,14 +6,14 @@ module Mechanize::ElementMatcher
         criteria = if String === criteria then
                      {:name => criteria}
                    else
-                     criteria.map do |k, v|
-                       k = :dom_id if k.to_sym == :id
-                       k = :dom_class if k.to_sym == :class
-                       [k, v]
-                     end
+                     Hash[criteria.map do |k, v|
+                            k = :dom_id if k.to_sym == :id
+                            k = :dom_class if k.to_sym == :class
+                            [k, v]
+                          end]
                    end
 
-        f = #{plural}.find_all do |thing|
+        f = select_#{plural}(criteria.delete(:search)).find_all do |thing|
           criteria.all? do |k,v|
             v === thing.send(k)
           end
@@ -28,6 +28,17 @@ module Mechanize::ElementMatcher
         f
       end
 
+      def select_#{plural} selector
+        if selector.nil? then
+          #{plural}
+        else
+          nodes = search(selector)
+          #{plural}.find_all do |element|
+            nodes.include?(element.node)
+          end
+        end
+      end
+
       alias :#{singular} :#{singular}_with
     CODE
   end
