@@ -49,17 +49,35 @@ class Mechanize::File
   end
 
   ##
-  # Use this method to save the content of this object to +filename+
+  # Use this method to save the content of this object to +filename+.
+  #
+  #   file.save 'index.html'
+  #   file.save 'index.html' # saves index.html.1
+  #   file.save 'index.html'
 
   def save filename = nil
     filename = find_free_name filename
+    save! filename
+  end
+
+  alias save_as save
+
+  ##
+  # Use this method to save the content of this object to +filename+.
+  # This method will overwrite any existing filename that exists with the
+  # same name.
+  #
+  #   file.save 'index.html'
+  #   file.save! 'index.html' # overwrite original file
+
+  def save! filename = nil
+    dirname = File.dirname filename
+    FileUtils.mkdir_p dirname
 
     open filename, 'wb' do |f|
       f.write body
     end
   end
 
-  alias save_as save
-
 end
 
