@@ -1,7 +1,7 @@
 require 'mechanize/element_matcher'
 
 # This class encapsulates a form parsed out of an HTML page.  Each type of
-# input fields available in a form can be accessed through this object.
+# input field available in a form can be accessed through this object.
 #
 # == Examples
 #
@@ -38,7 +38,7 @@ class Mechanize::Form
   attr_reader :form_node
   attr_reader :page
 
-  def initialize(node, mech=nil, page=nil)
+  def initialize(node, mech = nil, page = nil)
     @enctype = node['enctype'] || 'application/x-www-form-urlencoded'
     @form_node        = node
     @action           = Mechanize::Util.html_unescape(node['action'])
@@ -60,26 +60,75 @@ class Mechanize::Form
 
   alias :has_key? :has_field?
 
+  # Returns whether or not the form contains a field with +value+
   def has_value?(value)
     fields.find { |f| f.value == value }
   end
 
-  def keys; fields.map { |f| f.name }; end
+  # Returns all field names (keys) for this form
+  def keys
+    fields.map { |f| f.name }
+  end
 
-  def values; fields.map { |f| f.value }; end
+  # Returns all field values for this form
+  def values
+    fields.map { |f| f.value }
+  end
 
-  def submits  ; @submits   ||= buttons.select { |f| f.class == Submit   }; end
-  def resets   ; @resets    ||= buttons.select { |f| f.class == Reset    }; end
-  def texts    ; @texts     ||=  fields.select { |f| f.class == Text     }; end
-  def hiddens  ; @hiddens   ||=  fields.select { |f| f.class == Hidden   }; end
-  def textareas; @textareas ||=  fields.select { |f| f.class == Textarea }; end
-  def keygens  ; @keygens   ||=  fields.select { |f| f.class == Keygen   }; end
+  # Returns all buttons of type Submit
+  def submits
+    @submits ||= buttons.select { |f| f.class == Submit }
+  end
 
-  def submit_button?(button_name)   submits.find{|f| f.name == button_name}; end
-  def reset_button?(button_name)     resets.find{|f| f.name == button_name}; end
-  def text_field?(field_name)         texts.find{|f| f.name == field_name}; end
-  def hidden_field?(field_name)     hiddens.find{|f| f.name == field_name}; end
-  def textarea_field?(field_name) textareas.find{|f| f.name == field_name}; end
+  # Returns all buttons of type Reset
+  def resets
+    @resets ||= buttons.select { |f| f.class == Reset }
+  end
+
+  # Returns all fields of type Text
+  def texts
+    @texts ||= fields.select { |f| f.class == Text }
+  end
+
+  # Returns all fields of type Hidden
+  def hiddens
+    @hiddens ||= fields.select { |f| f.class == Hidden }
+  end
+
+  # Returns all fields of type Textarea
+  def textareas
+    @textareas ||= fields.select { |f| f.class == Textarea }
+  end
+
+  # Returns all fields of type Keygen
+  def keygens
+    @keygens ||= fields.select { |f| f.class == Keygen }
+  end
+
+  # Returns whether or not the form contains a Submit button named +button_name+
+  def submit_button?(button_name)
+    submits.find { |f| f.name == button_name }
+  end
+
+  # Returns whether or not the form contains a Reset button named +button_name+
+  def reset_button?(button_name)
+    resets.find { |f| f.name == button_name }
+  end
+
+  # Returns whether or not the form contains a Text field named +field_name+
+  def text_field?(field_name)
+    texts.find { |f| f.name == field_name }
+  end
+
+  # Returns whether or not the form contains a Hidden field named +field_name+
+  def hidden_field?(field_name)
+    hiddens.find { |f| f.name == field_name }
+  end
+
+  # Returns whether or not the form contains a Textarea named +field_name+
+  def textarea_field?(field_name)
+    textareas.find { |f| f.name == field_name }
+  end
 
   # This method is a shortcut to get form's DOM id.
   # Common usage:
@@ -117,7 +166,6 @@ class Mechanize::Form
   # following:
   #
   #   form.set_fields :foo => { 1 => 'bar' }
-
   def set_fields fields = {}
     fields.each do |name, v|
       case v
@@ -139,18 +187,14 @@ class Mechanize::Form
     end
   end
 
-  # Fetch the value of the first input field with the name passed in
-  # ==Example
-  # Fetch the value set in the input field 'name'
+  # Fetch the value of the first input field with the name passed in. Example:
   #  puts form['name']
   def [](field_name)
     f = field(field_name)
     f && f.value
   end
 
-  # Set the value of the first input field with the name passed in
-  # ==Example
-  # Set the value in the input field 'name' to "Aaron"
+  # Set the value of the first input field with the name passed in. Example:
   #  form['name'] = 'Aaron'
   def []=(field_name, value)
     f = field(field_name)
@@ -173,8 +217,9 @@ class Mechanize::Form
     super
   end
 
-  # Submit this form with the button passed in
-  def submit button=nil, headers = {}
+  # Submit the form. Does not include the +button+ as a form parameter.
+  # Use +click_button+ or provide button as a parameter.
+  def submit button = nil, headers = {}
     @mech.submit(self, button, headers)
   end
 
@@ -207,9 +252,13 @@ class Mechanize::Form
     query = []
     @mech.log.info("form encoding: #{encoding}") if @mech && @mech.log
 
+    save_hash_field_order
+
     successful_controls = []
 
-    (fields + checkboxes).sort.each do |f|
+    (fields + checkboxes).reject do |f|
+      f.node["disabled"]
+    end.sort.each do |f|
       case f
       when Mechanize::Form::CheckBox
         if f.checked
@@ -231,12 +280,14 @@ class Mechanize::Form
     radio_groups.each_value do |g|
       checked = g.select {|f| f.checked}
 
-      if checked.size == 1
-        f = checked.first
-        successful_controls << f
-      elsif checked.size > 1
+      if checked.uniq.size > 1 then
+        values = checked.map { |button| button.value }.join(', ').inspect
+        name = checked.first.name.inspect
         raise Mechanize::Error,
-              "multiple radiobuttons are checked in the same group!"
+              "radiobuttons #{values} are checked in the #{name} group, " \
+              "only one is allowed"
+      else
+        successful_controls << checked.first unless checked.empty?
       end
     end
 
@@ -252,6 +303,19 @@ class Mechanize::Form
     query
   end
 
+  # This method adds an index to all fields that have Hash nodes. This
+  # enables field sorting to maintain order.
+  def save_hash_field_order
+    index = 0
+
+    fields.each do |field|
+      if Hash === field.node
+        field.index = index
+        index += 1
+      end
+    end
+  end
+
   # This method adds a button to the query.  If the form needs to be
   # submitted with multiple buttons, pass each button to this method.
   def add_button_to_query(button)
@@ -479,7 +543,7 @@ class Mechanize::Form
       next if type == 'reset'
       @buttons << Button.new(node)
     end
-    
+
     # Find all keygen tags
     form_node.search('keygen').each do |node|
       @fields << Keygen.new(node, node['value'] || '')
@@ -530,7 +594,6 @@ class Mechanize::Form
 
     body
   end
-
 end
 
 require 'mechanize/form/field'
@@ -548,4 +611,3 @@ require 'mechanize/form/option'
 require 'mechanize/form/radio_button'
 require 'mechanize/form/check_box'
 require 'mechanize/form/select_list'
-
