@@ -43,11 +43,9 @@ class Mechanize::HTTP::Agent
 
   # :section: HTTP Authentication
 
+  attr_reader :auth_store # :nodoc:
   attr_reader :authenticate_methods # :nodoc:
   attr_reader :digest_challenges # :nodoc:
-  attr_accessor :user
-  attr_accessor :password
-  attr_accessor :domain
 
   # :section: Redirection
 
@@ -71,6 +69,12 @@ class Mechanize::HTTP::Agent
   # Maximum number of redirects to follow
   attr_accessor :redirection_limit
 
+  # :section: Allowed error codes
+
+  # List of error codes to handle without raising an exception.
+
+  attr_accessor :allowed_error_codes
+
   # :section: Robots
 
   # When true, this agent will consult the site's robots.txt for each access.
@@ -108,6 +112,11 @@ class Mechanize::HTTP::Agent
 
   attr_reader :http # :nodoc:
 
+  # When set to true mechanize will ignore an EOF during chunked transfer
+  # encoding so long as at least one byte was received.  Be careful when
+  # enabling this as it may cause data loss.
+  attr_accessor :ignore_bad_chunking
+
   # Handlers for various URI schemes
   attr_accessor :scheme_handlers
 
@@ -117,6 +126,7 @@ class Mechanize::HTTP::Agent
   # implementation detail of mechanize and its API may change at any time.
 
   def initialize
+    @allowed_error_codes      = []
     @conditional_requests     = true
     @context                  = nil
     @content_encoding_hooks   = []
@@ -125,6 +135,7 @@ class Mechanize::HTTP::Agent
     @follow_meta_refresh_self = false
     @gzip_enabled             = true
     @history                  = Mechanize::History.new
+    @ignore_bad_chunking      = false
     @keep_alive               = true
     @max_file_buffer          = 100_000 # 5MB for response bodies
     @open_timeout             = nil
@@ -139,6 +150,7 @@ class Mechanize::HTTP::Agent
     @webrobots                = nil
 
     # HTTP Authentication
+    @auth_store           = Mechanize::HTTP::AuthStore.new
     @authenticate_parser  = Mechanize::HTTP::WWWAuthenticateParser.new
     @authenticate_methods = Hash.new do |methods, uri|
       methods[uri] = Hash.new do |realms, auth_scheme|
@@ -147,9 +159,6 @@ class Mechanize::HTTP::Agent
     end
     @digest_auth          = Net::HTTP::DigestAuth.new
     @digest_challenges    = {}
-    @password             = nil # HTTP auth password
-    @user                 = nil # HTTP auth user
-    @domain               = nil # NTLM HTTP domain
 
     # SSL
     @pass = nil
@@ -170,6 +179,33 @@ class Mechanize::HTTP::Agent
     @http.keep_alive   = 300
   end
 
+  ##
+  # Adds credentials +user+, +pass+ for +uri+.  If +realm+ is set the
+  # credentials are used only for that realm.  If +realm+ is not set the
+  # credentials become the default for any realm on that URI.
+  #
+  # +domain+ and +realm+ are exclusive as NTLM does not follow RFC 2617.  If
+  # +domain+ is given it is only used for NTLM authentication.
+
+  def add_auth uri, user, password, realm = nil, domain = nil
+    @auth_store.add_auth uri, user, password, realm, domain
+  end
+
+  ##
+  # USE OF add_default_auth IS NOT RECOMMENDED AS IT MAY EXPOSE PASSWORDS TO
+  # THIRD PARTIES
+  #
+  # Adds credentials +user+, +pass+ as the default authentication credentials.
+  # If no other credentials are available  these will be returned from
+  # credentials_for.
+  #
+  # If +domain+ is given it is only used for NTLM authentication.
+
+  def add_default_auth user, password, domain = nil # :nodoc:
+    @auth_store.add_default_auth user, password, domain
+  end
+
+  ##
   # Retrieves +uri+ and parses it into a page or other object according to
   # PluggableParser.  If the URI is an HTTP or HTTPS scheme URI the given HTTP
   # +method+ is used to retrieve it, along with the HTTP +headers+, request
@@ -180,28 +216,22 @@ class Mechanize::HTTP::Agent
 
   def fetch uri, method = :get, headers = {}, params = [],
             referer = current_page, redirects = 0
+
     referer_uri = referer ? referer.uri : nil
-
-    uri = resolve uri, referer
-
+    uri         = resolve uri, referer
     uri, params = resolve_parameters uri, method, params
-
-    request = http_request uri, method, params
-
-    connection = connection_for uri
+    request     = http_request uri, method, params
+    connection  = connection_for uri
 
     request_auth             request, uri
-
     disable_keep_alive       request
     enable_gzip              request
-
     request_language_charset request
     request_cookies          request, uri
     request_host             request, uri
     request_referer          request, uri, referer_uri
     request_user_agent       request
     request_add_headers      request, headers
-
     pre_connect              request
 
     # Consult robots.txt
@@ -210,11 +240,9 @@ class Mechanize::HTTP::Agent
     end
 
     # Add If-Modified-Since if page is in history
-    page = visited_page(uri)
-
-    if (page = visited_page(uri)) and page.response['Last-Modified']
-      request['If-Modified-Since'] = page.response['Last-Modified']
-    end if(@conditional_requests)
+    if page = visited_page(uri) and last_modified = page.response['Last-Modified']
+      request['If-Modified-Since'] = last_modified
+    end if @conditional_requests
 
     # Specify timeouts if given
     connection.open_timeout = @open_timeout if @open_timeout
@@ -225,13 +253,20 @@ class Mechanize::HTTP::Agent
     response_body_io = nil
 
     # Send the request
-    response = connection.request(uri, request) { |res|
-      response_log res
+    begin
+      response = connection.request(uri, request) { |res|
+        response_log res
 
-      response_body_io = response_read res, request, uri
+        response_body_io = response_read res, request, uri
 
-      res
-    }
+        res
+      }
+    rescue Mechanize::ChunkedTerminationError => e
+      raise unless @ignore_bad_chunking
+
+      response = e.response
+      response_body_io = e.body_io
+    end
 
     hook_content_encoding response, uri, response_body_io
 
@@ -247,12 +282,12 @@ class Mechanize::HTTP::Agent
     meta = response_follow_meta_refresh response, uri, page, redirects
     return meta if meta
 
+    if robots && page.is_a?(Mechanize::Page)
+      page.parser.noindex? and raise Mechanize::RobotsDisallowedError.new(uri)
+    end
+
     case response
     when Net::HTTPSuccess
-      if robots && page.is_a?(Mechanize::Page)
-        page.parser.noindex? and raise Mechanize::RobotsDisallowedError.new(uri)
-      end
-
       page
     when Mechanize::FileResponse
       page
@@ -260,12 +295,16 @@ class Mechanize::HTTP::Agent
       log.debug("Got cached page") if log
       visited_page(uri) || page
     when Net::HTTPRedirection
-      response_redirect response, method, page, redirects, referer
+      response_redirect response, method, page, redirects, headers, referer
     when Net::HTTPUnauthorized
       response_authenticate(response, page, uri, request, headers, params,
                             referer)
     else
-      raise Mechanize::ResponseCodeError.new(page), "Unhandled response"
+      if @allowed_error_codes.any? {|code| code.to_s == page.code} then
+        page
+      else
+        raise Mechanize::ResponseCodeError.new(page, 'unhandled response')
+      end
     end
   end
 
@@ -308,10 +347,12 @@ class Mechanize::HTTP::Agent
     @history.last
   end
 
+  # Returns the maximum size for the history stack.
   def max_history
     @history.max_size
   end
 
+  # Set the maximum size for the history stack.
   def max_history=(length)
     @history.max_size = length
   end
@@ -367,6 +408,11 @@ class Mechanize::HTTP::Agent
     end
   end
 
+  # Closes all open connections for this agent.
+  def shutdown
+    http.shutdown
+  end
+
   ##
   # Decodes a gzip-encoded +body_io+.  If it cannot be decoded, inflate is
   # tried followed by raising an error.
@@ -379,20 +425,23 @@ class Mechanize::HTTP::Agent
     zio.finish
 
     return out_io
-  rescue Zlib::Error
-    log.error('unable to gunzip response, trying raw inflate') if log
+  rescue Zlib::Error => gz_error
+    log.warn "unable to gunzip response: #{gz_error} (#{gz_error.class})" if
+      log
 
     body_io.rewind
     body_io.read 10
 
     begin
+      log.warn "trying raw inflate on response" if log
       return inflate body_io, -Zlib::MAX_WBITS
     rescue Zlib::Error => e
-      log.error("unable to gunzip response: #{e}") if log
+      log.error "unable to inflate response: #{e} (#{e.class})" if log
       raise
     end
   ensure
-    zio.close if zio and not zio.closed?
+    # do not close a second time if we failed the first time
+    zio.close if zio and !(zio.closed? or gz_error)
     body_io.close unless body_io.closed?
   end
 
@@ -470,16 +519,18 @@ class Mechanize::HTTP::Agent
       request_auth_digest request, uri, realm, base_uri, false
     elsif realm = schemes[:iis_digest].find { |r| r.uri == base_uri } then
       request_auth_digest request, uri, realm, base_uri, true
-    elsif schemes[:basic].find { |r| r.uri == base_uri } then
-      request.basic_auth @user, @password
+    elsif realm = schemes[:basic].find { |r| r.uri == base_uri } then
+      user, password, = @auth_store.credentials_for uri, realm.realm
+      request.basic_auth user, password
     end
   end
 
   def request_auth_digest request, uri, realm, base_uri, iis
     challenge = @digest_challenges[realm]
 
-    uri.user = @user
-    uri.password = @password
+    user, password, = @auth_store.credentials_for uri, realm.realm
+    uri.user     = user
+    uri.password = password
 
     auth = @digest_auth.auth_header uri, challenge.to_s, request.method, iis
     request['Authorization'] = auth
@@ -523,8 +574,8 @@ class Mechanize::HTTP::Agent
   # major browsers do.
   def request_referer request, uri, referer
     return unless referer
-    return if 'https' == referer.scheme.downcase and
-              'https' != uri.scheme.downcase
+    return if 'https'.casecmp(referer.scheme) == 0 and
+              'https'.casecmp(uri.scheme) != 0
     if referer.fragment || referer.user || referer.password
       referer = referer.dup
       referer.fragment = referer.user = referer.password = nil
@@ -558,7 +609,13 @@ class Mechanize::HTTP::Agent
         if RUBY_VERSION >= "1.9.0"
           Mechanize::Util.uri_escape(match)
         else
-          sprintf('%%%X', match.unpack($KCODE == 'UTF8' ? 'U' : 'C')[0])
+          begin
+            sprintf('%%%X', match.unpack($KCODE == 'UTF8' ? 'U' : 'C').first)
+          rescue ArgumentError
+            # workaround for ruby 1.8 with -Ku but ISO-8859-1 characters in
+            # URIs.  See #227.  I can't wait to drop 1.8 support
+            sprintf('%%%X', match.unpack('C').first)
+          end
         end
       }
 
@@ -575,6 +632,8 @@ class Mechanize::HTTP::Agent
       end
     end
 
+    uri.host = referer_uri.host if referer_uri && URI::HTTP === uri && uri.host.nil?
+
     scheme = uri.relative? ? 'relative' : uri.scheme.downcase
     uri = @scheme_handlers[scheme].call(uri, referer)
 
@@ -597,7 +656,16 @@ class Mechanize::HTTP::Agent
         base = nil
       end
 
-      uri = referer_uri + (base ? base.uri : referer_uri) + uri
+      base = referer_uri + (base ? base.uri : referer_uri)
+
+      # Workaround for URI's bug in that it squashes consecutive
+      # slashes.  See #304.
+      if uri.path.match(%r{\A(.*?/)(?!/\.\.?(?!/))(/.*)\z}i)
+        uri = URI((base + $1).to_s + $2)
+      else
+        uri = base + uri
+      end
+
       # Strip initial "/.." bits from the path
       uri.path.sub!(/^(\/\.\.)+(?=\/)/, '')
     end
@@ -643,14 +711,20 @@ class Mechanize::HTTP::Agent
 
   def response_authenticate(response, page, uri, request, headers, params,
                             referer)
-    raise Mechanize::UnauthorizedError, page unless @user || @password
-
     www_authenticate = response['www-authenticate']
 
-    raise Mechanize::UnauthorizedError, page unless www_authenticate
+    unless www_authenticate = response['www-authenticate'] then
+      message = 'WWW-Authenticate header missing in response'
+      raise Mechanize::UnauthorizedError.new(page, nil, message)
+    end
 
     challenges = @authenticate_parser.parse www_authenticate
 
+    unless @auth_store.credentials? uri, challenges then
+      message = "no credentials found, provide some with #add_auth"
+      raise Mechanize::UnauthorizedError.new(page, challenges, message)
+    end
+
     if challenge = challenges.find { |c| c.scheme =~ /^Digest$/i } then
       realm = challenge.realm uri
 
@@ -662,23 +736,30 @@ class Mechanize::HTTP::Agent
 
       existing_realms = @authenticate_methods[realm.uri][auth_scheme]
 
-      raise Mechanize::UnauthorizedError, page if
-        existing_realms.include? realm
+      if existing_realms.include? realm
+        message = 'Digest authentication failed'
+        raise Mechanize::UnauthorizedError.new(page, challeges, message)
+      end
 
       existing_realms << realm
       @digest_challenges[realm] = challenge
     elsif challenge = challenges.find { |c| c.scheme == 'NTLM' } then
       existing_realms = @authenticate_methods[uri + '/'][:ntlm]
 
-      raise Mechanize::UnauthorizedError, page if
-        existing_realms.include?(realm) and not challenge.params
+      if existing_realms.include?(realm) and not challenge.params then
+        message = 'NTLM authentication failed'
+        raise Mechanize::UnauthorizedError.new(page, challenges, message)
+      end
 
       existing_realms << realm
 
       if challenge.params then
         type_2 = Net::NTLM::Message.decode64 challenge.params
 
-        type_3 = type_2.response({ :user => @user, :password => @password, :domain => @domain },
+        user, password, domain = @auth_store.credentials_for uri, nil
+
+        type_3 = type_2.response({ :user => user, :password => password,
+                                   :domain => domain },
                                  { :ntlmv2 => true }).encode64
 
         headers['Authorization'] = "NTLM #{type_3}"
@@ -691,12 +772,15 @@ class Mechanize::HTTP::Agent
 
       existing_realms = @authenticate_methods[realm.uri][:basic]
 
-      raise Mechanize::UnauthorizedError, page if
-        existing_realms.include? realm
+      if existing_realms.include? realm then
+        message = 'Basic authentication failed'
+        raise Mechanize::UnauthorizedError.new(page, challenges, message)
+      end
 
       existing_realms << realm
     else
-      raise Mechanize::UnauthorizedError, page
+      message = 'unsupported authentication scheme'
+      raise Mechanize::UnauthorizedError.new(page, challenges, message)
     end
 
     fetch uri, request.method.downcase.to_sym, headers, params, referer
@@ -714,7 +798,7 @@ class Mechanize::HTTP::Agent
     return body_io if length.zero?
 
     out_io = case response['Content-Encoding']
-             when nil, 'none', '7bit' then
+             when nil, 'none', '7bit', "" then
                body_io
              when 'deflate' then
                content_encoding_inflate body_io
@@ -737,7 +821,7 @@ class Mechanize::HTTP::Agent
     begin
       if Tempfile === body_io and
          (StringIO === out_io or out_io.path != body_io.path) then
-        body_io.close! 
+        body_io.close!
       end
     rescue IOError
       # HACK ruby 1.8 raises IOError when closing the stream
@@ -761,14 +845,15 @@ class Mechanize::HTTP::Agent
   end
 
   def save_cookies(uri, set_cookie)
-    log = log()	 # reduce method calls
-    Mechanize::Cookie.parse(uri, set_cookie, log) { |c|
-      if @cookie_jar.add(uri, c)
-        log.debug("saved cookie: #{c}") if log
-      else
-        log.debug("rejected cookie: #{c}") if log
-      end
-    }
+    return [] if set_cookie.nil?
+    if log = log()	 # reduce method calls
+      @cookie_jar.parse(set_cookie, uri, :logger => log) { |c|
+        log.debug("saved cookie: #{c}")
+        true
+      }
+    else
+      @cookie_jar.parse(set_cookie, uri)
+    end
   end
 
   def response_follow_meta_refresh response, uri, page, redirects
@@ -782,7 +867,7 @@ class Mechanize::HTTP::Agent
     sleep delay
     @history.push(page, page.uri)
     fetch new_url, :get, {}, [],
-          Mechanize::Page.new, redirects
+          Mechanize::Page.new, redirects + 1
   end
 
   def response_log response
@@ -804,9 +889,7 @@ class Mechanize::HTTP::Agent
     content_length = response.content_length
 
     if use_tempfile? content_length then
-      body_io = Tempfile.new 'mechanize-raw'
-      body_io.unlink
-      body_io.binmode if defined? body_io.binmode
+      body_io = make_tempfile 'mechanize-raw'
     else
       body_io = StringIO.new
     end
@@ -819,9 +902,7 @@ class Mechanize::HTTP::Agent
         total += part.length
 
         if StringIO === body_io and use_tempfile? total then
-          new_io = Tempfile.new 'mechanize-raw'
-          new_io.unlink
-          new_io.binmode
+          new_io = make_tempfile 'mechanize-raw'
 
           new_io.write body_io.string
 
@@ -831,7 +912,14 @@ class Mechanize::HTTP::Agent
         body_io.write(part)
         log.debug("Read #{part.length} bytes (#{total} total)") if log
       }
-    rescue Net::HTTP::Persistent::Error => e
+    rescue EOFError => e
+      # terminating CRLF might be missing, let the user check the document
+      raise unless response.chunked? and total.nonzero?
+
+      body_io.rewind
+      raise Mechanize::ChunkedTerminationError.new(e, response, body_io, uri,
+                                                   @context)
+    rescue Net::HTTP::Persistent::Error, Errno::ECONNRESET => e
       body_io.rewind
       raise Mechanize::ResponseReadError.new(e, response, body_io, uri,
                                              @context)
@@ -840,21 +928,25 @@ class Mechanize::HTTP::Agent
     body_io.flush
     body_io.rewind
 
-    raise Mechanize::ResponseCodeError, response if
+    raise Mechanize::ResponseCodeError.new(response, uri) if
       Net::HTTPUnknownResponse === response
 
     content_length = response.content_length
 
     unless Net::HTTP::Head === request or Net::HTTPRedirection === response then
-      raise EOFError, "Content-Length (#{content_length}) does not match " \
-                      "response body length (#{body_io.length})" if
-        content_length and content_length != body_io.length
+      if content_length and content_length != body_io.length
+        err = EOFError.new("Content-Length (#{content_length}) does not " \
+                      "match response body length (#{body_io.length})")
+        raise Mechanize::ResponseReadError.new(err, response, body_io, uri,
+                                                @context)
+      end
     end
 
     body_io
   end
 
-  def response_redirect response, method, page, redirects, referer = current_page
+  def response_redirect(response, method, page, redirects, headers,
+                        referer = current_page)
     case @redirect_ok
     when true, :all
       # shortcut
@@ -871,10 +963,15 @@ class Mechanize::HTTP::Agent
 
     redirect_method = method == :head ? :head : :get
 
+    # Make sure we are not copying over the POST headers from the original request
+    ['Content-Length', 'Content-MD5', 'Content-Type'].each do |key|
+      headers.delete key
+    end
+
     @history.push(page, page.uri)
     new_uri = resolve response['Location'].to_s, page
 
-    fetch new_uri, redirect_method, {}, [], referer, redirects + 1
+    fetch new_uri, redirect_method, headers, [], referer, redirects + 1
   end
 
   # :section: Robots
@@ -1048,10 +1145,7 @@ class Mechanize::HTTP::Agent
 
     until input_io.eof? do
       if StringIO === out_io and use_tempfile? out_io.size then
-        new_io = Tempfile.new name
-        new_io.unlink
-        new_io.binmode
-
+        new_io = make_tempfile name
         new_io.write out_io.string
         out_io = new_io
       end
@@ -1074,11 +1168,11 @@ class Mechanize::HTTP::Agent
       inflate.inflate chunk
     end
 
-    out_io.write inflate.finish
+    inflate.finish
 
     out_io
   ensure
-    inflate.close
+    inflate.close if inflate.finished?
   end
 
   def log
@@ -1117,6 +1211,13 @@ class Mechanize::HTTP::Agent
     @http.proxy = proxy_uri
   end
 
+  def make_tempfile name
+    io = Tempfile.new name
+    io.unlink
+    io.binmode if io.respond_to? :binmode
+    io
+  end
+
   def use_tempfile? size
     return false unless @max_file_buffer
     return false unless size
@@ -1124,5 +1225,12 @@ class Mechanize::HTTP::Agent
     size >= @max_file_buffer
   end
 
+  def reset
+    @cookie_jar.clear
+    @history.clear
+  end
+
 end
 
+require 'mechanize/http/auth_store'
+
