@@ -1,6 +1,7 @@
 require 'mechanize/file'
 require 'mechanize/file_saver'
 require 'mechanize/page'
+require 'mechanize/xml_file'
 
 ##
 # Mechanize allows different parsers for different content types.  Mechanize
@@ -64,7 +65,7 @@ class Mechanize::PluggableParser
     :xhtml => 'application/xhtml+xml',
     :pdf   => 'application/pdf',
     :csv   => 'text/csv',
-    :xml   => 'text/xml',
+    :xml   => ['text/xml', 'application/xml'],
   }
 
   attr_accessor :default
@@ -74,7 +75,9 @@ class Mechanize::PluggableParser
       CONTENT_TYPES[:html]  => Mechanize::Page,
       CONTENT_TYPES[:xhtml] => Mechanize::Page,
       CONTENT_TYPES[:wap]   => Mechanize::Page,
-      'image'               => Mechanize::Image
+      'image'               => Mechanize::Image,
+      'text/xml'            => Mechanize::XmlFile,
+      'application/xml'     => Mechanize::XmlFile,
     }
 
     @default = Mechanize::File
@@ -138,7 +141,9 @@ class Mechanize::PluggableParser
   # Registers +klass+ as the parser for text/xml content
 
   def xml=(klass)
-    register_parser(CONTENT_TYPES[:xml], klass)
+    CONTENT_TYPES[:xml].each do |content_type|
+      register_parser content_type, klass
+    end
   end
 
   ##
