@@ -41,9 +41,17 @@ class TestMechanize < Mechanize::TestCase
   end
 
   def test_basic_auth
-    @mech.basic_auth('user', 'pass')
-    page = @mech.get("http://localhost/basic_auth")
-    assert_equal('You are authenticated', page.body)
+    _, err = capture_io do
+      @mech.basic_auth 'user', 'pass' # warns
+    end
+
+    line = __LINE__ - 3
+    file = File.basename __FILE__
+
+    assert_match "#{file} line #{line}", err
+
+    page = @mech.get @uri + '/basic_auth'
+    assert_equal 'You are authenticated', page.body
   end
 
   def test_cert_key_file
@@ -68,6 +76,8 @@ class TestMechanize < Mechanize::TestCase
     end
 
     assert_equal ssl_certificate, mech.certificate
+    assert_equal ssl_certificate, mech.cert
+    assert_equal ssl_private_key, mech.key
   end
 
   def test_cert_store
@@ -90,6 +100,18 @@ class TestMechanize < Mechanize::TestCase
                  @mech.history.last.uri.to_s)
   end
 
+  def test_click_bogus_link_with_cookies
+    @mech.cookie_jar = cookie_jar("a=b")
+
+    page = html_page <<-BODY
+<a href="http:///index.html">yes really</a>
+    BODY
+
+    page.links[0].click
+
+    assert_equal '/index.html', requests.first.path
+  end
+
   def test_click_frame
     frame = node 'frame', 'src' => '/index.html'
     frame = Mechanize::Page::Frame.new frame, @mech, fake_page
@@ -241,10 +263,8 @@ but not <a href="/" rel="me nofollow">this</a>!
 
   def test_cookies
     uri = URI 'http://example'
-    jar = Mechanize::CookieJar.new
-    Mechanize::Cookie.parse uri, 'a=b' do |cookie|
-      jar.add uri, cookie
-    end
+    jar = HTTP::CookieJar.new
+    jar.parse 'a=b', uri
 
     @mech.cookie_jar = jar
 
@@ -254,7 +274,7 @@ but not <a href="/" rel="me nofollow">this</a>!
   def test_cookie_jar
     assert_kind_of Mechanize::CookieJar, @mech.cookie_jar
 
-    jar = Mechanize::CookieJar.new
+    jar = HTTP::CookieJar.new
 
     @mech.cookie_jar = jar
 
@@ -326,7 +346,11 @@ but not <a href="/" rel="me nofollow">this</a>!
 
   def test_get_HTTP
     page = @mech.get('HTTP://localhost/', { :q => 'hello' })
-    assert_equal('HTTP://localhost/?q=hello', page.uri.to_s)
+
+    assert_kind_of URI::HTTP, page.uri
+    assert_equal 'localhost', page.uri.host
+    assert_equal 80,          page.uri.port
+    assert_equal '/?q=hello', page.uri.request_uri
   end
 
   def test_get_anchor
@@ -340,19 +364,19 @@ but not <a href="/" rel="me nofollow">this</a>!
     end
   end
 
-  def test_get_basic_auth_bad
-    @mech.basic_auth('aaron', 'aaron')
+  def test_get_auth_bad
+    @mech.add_auth(@uri, 'aaron', 'aaron')
 
     e = assert_raises Mechanize::UnauthorizedError do
-      @mech.get("http://localhost/basic_auth")
+      @mech.get(@uri + '/basic_auth')
     end
 
     assert_equal("401", e.response_code)
   end
 
-  def test_get_basic_auth_none
+  def test_get_auth_none
     e = assert_raises Mechanize::UnauthorizedError do
-      @mech.get("http://localhost/basic_auth")
+      @mech.get(@uri + '/basic_auth')
     end
 
     assert_equal("401", e.response_code)
@@ -372,20 +396,11 @@ but not <a href="/" rel="me nofollow">this</a>!
   end
 
   def test_get_digest_auth
-    block_called = false
+    @mech.add_auth @uri, 'user', 'pass'
 
-    @mech.basic_auth('user', 'pass')
+    page = @mech.get @uri + '/digest_auth'
 
-    @mech.pre_connect_hooks << lambda { |_, request|
-      block_called = true
-      request.to_hash.each do |k,v|
-        assert_equal(1, v.length)
-      end
-    }
-
-    page = @mech.get("http://localhost/digest_auth")
-    assert_equal('You are authenticated', page.body)
-    assert block_called
+    assert_equal 'You are authenticated', page.body
   end
 
   def test_get_follow_meta_refresh
@@ -399,6 +414,21 @@ but not <a href="/" rel="me nofollow">this</a>!
                  @mech.history.first.uri.to_s)
     assert_equal('http://localhost/index.html', page.uri.to_s)
     assert_equal('http://localhost/index.html', @mech.history.last.uri.to_s)
+
+    [5, 6].each { |limit|
+      @mech.redirection_limit = limit
+      begin
+        @mech.get('http://localhost/tc_follow_meta_loop_1.html')
+      rescue => e
+        assert_instance_of Mechanize::RedirectLimitReachedError, e
+        assert_equal limit, e.redirects
+        if limit % 2 == 0
+          assert_equal '/tc_follow_meta_loop_1.html', e.page.uri.path
+        else
+          assert_equal '/tc_follow_meta_loop_2.html', e.page.uri.path
+        end
+      end
+    }
   end
 
   def test_get_follow_meta_refresh_anywhere
@@ -422,15 +452,15 @@ but not <a href="/" rel="me nofollow">this</a>!
     @mech.follow_meta_refresh = true
     @mech.follow_meta_refresh_self = true
 
-    page = @mech.get('http://localhost/refresh_with_empty_url')
+    page = @mech.get('http://example/refresh_with_empty_url')
 
     assert_equal(3, @mech.history.length)
-    assert_equal('http://localhost/refresh_with_empty_url',
+    assert_equal('http://example/refresh_with_empty_url',
                  @mech.history[0].uri.to_s)
-    assert_equal('http://localhost/refresh_with_empty_url',
+    assert_equal('http://example/refresh_with_empty_url',
                  @mech.history[1].uri.to_s)
-    assert_equal('http://localhost/index.html', page.uri.to_s)
-    assert_equal('http://localhost/index.html', @mech.history.last.uri.to_s)
+    assert_equal('http://example/', page.uri.to_s)
+    assert_equal('http://example/', @mech.history.last.uri.to_s)
   end
 
   def test_get_follow_meta_refresh_in_body
@@ -448,15 +478,15 @@ but not <a href="/" rel="me nofollow">this</a>!
     @mech.follow_meta_refresh = true
     @mech.follow_meta_refresh_self = true
 
-    page = @mech.get('http://localhost/refresh_without_url')
+    page = @mech.get('http://example/refresh_without_url')
 
     assert_equal(3, @mech.history.length)
-    assert_equal('http://localhost/refresh_without_url',
+    assert_equal('http://example/refresh_without_url',
                  @mech.history[0].uri.to_s)
-    assert_equal('http://localhost/refresh_without_url',
+    assert_equal('http://example/refresh_without_url',
                  @mech.history[1].uri.to_s)
-    assert_equal('http://localhost/index.html', page.uri.to_s)
-    assert_equal('http://localhost/index.html', @mech.history.last.uri.to_s)
+    assert_equal('http://example/', page.uri.to_s)
+    assert_equal('http://example/', @mech.history.last.uri.to_s)
   end
 
   def test_get_follow_meta_refresh_referer_not_sent
@@ -554,9 +584,9 @@ but not <a href="/" rel="me nofollow">this</a>!
       requests << request
     }
 
-    page = @mech.get('http://localhost/http_refresh?refresh_time=0')
+    page = @mech.get('http://example/http_refresh?refresh_time=0')
 
-    assert_equal('http://localhost/index.html', page.uri.to_s)
+    assert_equal('http://example/', page.uri.to_s)
     assert_equal(2, @mech.history.length)
     assert_nil requests.last['referer']
   end
@@ -869,15 +899,15 @@ but not <a href="/" rel="me nofollow">this</a>!
     assert_equal "gender=female", requests.first.body
   end
 
-  def test_post_basic_auth
+  def test_post_auth
     requests = []
 
     @mech.pre_connect_hooks << proc { |agent, request|
       requests << request.class
     }
 
-    @mech.basic_auth('user', 'pass')
-    page = @mech.post("http://localhost/basic_auth")
+    @mech.add_auth(@uri, 'user', 'pass')
+    page = @mech.post(@uri + '/basic_auth')
     assert_equal('You are authenticated', page.body)
     assert_equal(2, requests.length)
     r1 = requests[0]
@@ -997,6 +1027,38 @@ but not <a href="/" rel="me nofollow">this</a>!
     assert_equal URI('http://user:pass@localhost:8080'), http.proxy_uri
   end
 
+  def test_shutdown
+    uri = URI 'http://localhost'
+    jar = HTTP::CookieJar.new
+    jar.parse 'a=b', uri
+
+    @mech.cookie_jar = jar
+
+    @mech.get("http://localhost/")
+
+    assert_match(/Hello World/, @mech.current_page.body)
+    refute_empty @mech.cookies
+    refute_empty Thread.current[@mech.agent.http.request_key]
+
+    @mech.shutdown
+
+    assert_nil Thread.current[@mech.agent.http.request_key]
+    assert_empty @mech.history
+    assert_empty @mech.cookies
+  end
+
+  def test_start
+    body, id = nil
+
+    Mechanize.start do |m|
+      body = m.get("http://localhost/").body
+      id = m.agent.http.request_key
+    end
+
+    assert_match(/Hello World/, body)
+    assert_nil Thread.current[id]
+  end
+
   def test_submit_bad_form_method
     page = @mech.get("http://localhost/bad_form_test.html")
     assert_raises ArgumentError do
