@@ -16,7 +16,18 @@ class TestMechanizeFile < Mechanize::TestCase
       Dir.chdir dir do
         page.save 'test.html'
 
+        assert File.exist? 'test.html'
         assert_equal '0123456789', File.read('test.html')
+
+        page.save 'test.html'
+
+        assert File.exist? 'test.html.1'
+        assert_equal '0123456789', File.read('test.html.1')
+
+        page.save 'test.html'
+
+        assert File.exist? 'test.html.2'
+        assert_equal '0123456789', File.read('test.html.2')
       end
     end
   end
@@ -64,5 +75,23 @@ class TestMechanizeFile < Mechanize::TestCase
     assert_equal "test.html", page.filename
   end
 
+  def test_save_overwrite
+    uri = URI 'http://example/test.html'
+    page = Mechanize::File.new uri, nil, ''
+
+    Dir.mktmpdir do |dir|
+      Dir.chdir dir do
+        page.save 'test.html'
+
+        assert File.exist? 'test.html'
+
+        page.save! 'test.html'
+
+        assert File.exist? 'test.html'
+        refute File.exist? 'test.html.1'
+      end
+    end
+  end
+
 end
 
