@@ -8,7 +8,7 @@ class TestMechanizeForm < Mechanize::TestCase
     @uri = URI 'http://example'
     @page = page @uri
 
-    @form = Mechanize::Form.new node('form', 'name' => __name__), @mech, @page
+    @form = Mechanize::Form.new node('form', 'name' => @NAME), @mech, @page
   end
 
   def test_action
@@ -25,7 +25,7 @@ class TestMechanizeForm < Mechanize::TestCase
     end
 
     assert_equal "#{button.inspect} does not belong to the same page " \
-                 "as the form \"#{@__name__}\" in #{@uri}",
+                 "as the form \"#{@NAME}\" in #{@uri}",
                  e.message
   end
 
@@ -65,6 +65,40 @@ class TestMechanizeForm < Mechanize::TestCase
     assert query.all? { |x| x[1] == '' }
   end
 
+  def test_build_query_radio_button_duplicate
+    html = Nokogiri::HTML <<-HTML
+<form>
+  <input type=radio name=name value=a checked=true>
+  <input type=radio name=name value=a checked=true>
+</form>
+    HTML
+
+    form = Mechanize::Form.new html.at('form'), @mech, @page
+
+    query = form.build_query
+
+    assert_equal [%w[name a]], query
+  end
+
+  def test_build_query_radio_button_multiple_checked
+    html = Nokogiri::HTML <<-HTML
+<form>
+  <input type=radio name=name value=a checked=true>
+  <input type=radio name=name value=b checked=true>
+</form>
+    HTML
+
+    form = Mechanize::Form.new html.at('form'), @mech, @page
+
+    e = assert_raises Mechanize::Error do
+      form.build_query
+    end
+
+    assert_equal 'radiobuttons "a, b" are checked in the "name" group, ' \
+                 'only one is allowed',
+                 e.message
+  end
+
   def test_method_missing_get
     page = html_page <<-BODY
 <form>
@@ -278,7 +312,7 @@ class TestMechanizeForm < Mechanize::TestCase
     page = html_page '<form><input name="a" value="b"><input name="a"></form>'
     form = page.forms.first
 
-    form.set_fields :a => ['c', 1] 
+    form.set_fields :a => ['c', 1]
 
     assert_equal 'b', form.fields.first.value
     assert_equal 'c', form.fields.last.value
@@ -887,4 +921,46 @@ class TestMechanizeForm < Mechanize::TestCase
     assert_equal expected, query
   end
 
+  def test_form_input_disabled
+    page = html_page <<-BODY
+<form name="form1" method="post" action="/form_post">
+  <input type="text" name="opa" value="omg" disabled />
+  <input type="submit" value="Submit" />
+</form>
+    BODY
+    form = page.forms.first
+
+    page = @mech.submit form
+
+    assert_empty page.links
+  end
+
+  def test_form_built_from_array_post
+    submitted = @mech.post(
+      'http://example/form_post',
+      [
+        %w[order_matters 0],
+        %w[order_matters 1],
+        %w[order_matters 2],
+        %w[mess_it_up asdf]
+      ]
+    )
+
+    assert_equal 'order_matters=0&order_matters=1&order_matters=2&mess_it_up=asdf', submitted.parser.at('#query').text
+  end
+
+  def test_form_built_from_hashes_submit
+    uri = URI 'http://example/form_post'
+    page = page uri
+    form = Mechanize::Form.new node('form', 'name' => @NAME, 'method' => 'POST'), @mech, page
+    form.fields << Mechanize::Form::Field.new({'name' => 'order_matters'}, '0')
+    form.fields << Mechanize::Form::Field.new({'name' => 'order_matters'}, '1')
+    form.fields << Mechanize::Form::Field.new({'name' => 'order_matters'}, '2')
+    form.fields << Mechanize::Form::Field.new({'name' => 'mess_it_up'}, 'asdf')
+
+    submitted = @mech.submit form
+
+    assert_equal 'order_matters=0&order_matters=1&order_matters=2&mess_it_up=asdf', submitted.parser.at('#query').text
+  end
+
 end
