@@ -16,7 +16,9 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     @res.instance_variable_set :@code, 200
     @res.instance_variable_set :@header, {}
 
-    @headers = if RUBY_VERSION > '1.9' then
+    @headers = if RUBY_VERSION >= '2.0.0' then
+                 %w[accept accept-encoding user-agent]
+               elsif RUBY_VERSION >= '1.9.0' then
                  %w[accept user-agent]
                else
                  %w[accept]
@@ -37,7 +39,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
       input_io.write '12345'
       input_io.rewind
 
-      out_io = @agent.auto_io __name__, 1024, input_io
+      out_io = @agent.auto_io @NAME, 1024, input_io
 
       assert_equal '12345', out_io.string
 
@@ -54,7 +56,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
       input_io.write '12345'
       input_io.rewind
 
-      @agent.auto_io __name__, 1, input_io do |chunk|
+      @agent.auto_io @NAME, 1, input_io do |chunk|
         chunks << chunk
       end
 
@@ -70,7 +72,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
       input_io.write '12345'
       input_io.rewind
 
-      out_io = @agent.auto_io __name__, 1, input_io
+      out_io = @agent.auto_io @NAME, 1, input_io
 
       result = out_io.read
       assert_equal '12345', result
@@ -86,7 +88,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
       input_io.write '12345'
       input_io.rewind
 
-      out_io = @agent.auto_io __name__, 1024, input_io do |chunk|
+      out_io = @agent.auto_io @NAME, 1024, input_io do |chunk|
         "x#{chunk}"
       end
 
@@ -158,13 +160,13 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     in_tmpdir do
       nonexistent = File.join Dir.pwd, 'nonexistent'
 
-      uri = URI.parse "file://#{nonexistent}"
+      uri = URI.parse "file:///#{nonexistent}"
 
       e = assert_raises Mechanize::ResponseCodeError do
         @agent.fetch uri
       end
 
-      assert_equal '404 => Net::HTTPNotFound', e.message
+      assert_match "404 => Net::HTTPNotFound for #{uri}", e.message
     end
   end
 
@@ -218,6 +220,14 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     @agent.fetch URI 'http://example/index.html'
   end
 
+  def test_fetch_ignore_bad_chunking
+    @agent.ignore_bad_chunking = true
+
+    file = @agent.fetch 'http://example/bad_chunking'
+
+    assert_equal '0123456789', file.content
+  end
+
   def test_fetch_post_connect_hook
     response = nil
     @agent.post_connect_hooks << lambda { |_, _, res, _| response = res }
@@ -227,6 +237,14 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert response
   end
 
+  def test_fetch_redirect_header
+    page = @agent.fetch('http://example/redirect', :get,
+                        'X-Location' => '/http_headers',
+                        'Range' => 'bytes=0-99999')
+
+    assert_match 'range|bytes=0-999', page.body
+  end
+
   def test_fetch_server_error
     e = assert_raises Mechanize::ResponseCodeError do
       @mech.get 'http://localhost/response_code?code=500'
@@ -235,6 +253,22 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal '500', e.response_code
   end
 
+  def test_fetch_allowed_error_codes
+    @agent.allowed_error_codes = ['500']
+
+    page = @mech.get 'http://localhost/response_code?code=500'
+
+    assert_equal '500', page.code
+  end
+
+  def test_fetch_allowed_error_codes_int
+    @agent.allowed_error_codes = [500]
+
+    page = @mech.get 'http://localhost/response_code?code=500'
+
+    assert_equal '500', page.code
+  end
+
   def test_get_meta_refresh_header_follow_self
     @agent.follow_meta_refresh = true
     @agent.follow_meta_refresh_self = true
@@ -436,8 +470,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
   end
 
   def test_request_auth_basic
-    @agent.user = 'user'
-    @agent.password = 'password'
+    @agent.add_auth @uri, 'user', 'password'
 
     auth_realm @uri, 'Basic', :basic
 
@@ -447,8 +480,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
   end
 
   def test_request_auth_digest
-    @agent.user = 'user'
-    @agent.password = 'password'
+    @agent.add_auth @uri, 'user', 'password'
 
     realm = auth_realm @uri, 'Digest', :digest
     @agent.digest_challenges[realm] = 'Digest realm=r, qop="auth"'
@@ -460,8 +492,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
   end
 
   def test_request_auth_iis_digest
-    @agent.user = 'user'
-    @agent.password = 'password'
+    @agent.add_auth @uri, 'user', 'password'
 
     realm = auth_realm @uri, 'Digest', :digest
     @agent.digest_challenges[realm] = 'Digest realm=r, qop="auth"'
@@ -480,27 +511,24 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
   def test_request_cookies
     uri = URI.parse 'http://host.example.com'
-    Mechanize::Cookie.parse uri, 'hello=world domain=.example.com' do |cookie|
-      @agent.cookie_jar.add uri, cookie
-    end
+    @agent.cookie_jar.parse 'hello=world domain=.example.com', uri
 
     @agent.request_cookies @req, uri
 
-    assert_equal 'hello=world domain=.example.com', @req['Cookie']
+    assert_equal 'hello="world domain=.example.com"', @req['Cookie']
   end
 
   def test_request_cookies_many
     uri = URI.parse 'http://host.example.com'
     cookie_str = 'a=b domain=.example.com, c=d domain=.example.com'
-    Mechanize::Cookie.parse uri, cookie_str do |cookie|
-      @agent.cookie_jar.add uri, cookie
-    end
+    @agent.cookie_jar.parse cookie_str, uri
 
     @agent.request_cookies @req, uri
 
-    expected = cookie_str.sub ', ', '; '
+    expected_variant1 = /a="b domain=\.example\.com"; c="d domain=\.example\.com"/
+    expected_variant2 = /c="d domain=\.example\.com"; a="b domain=\.example\.com"/
 
-    assert_equal expected, @req['Cookie']
+    assert_match(/^(#{expected_variant1}|#{expected_variant2})$/, @req['Cookie'])
   end
 
   def test_request_cookies_none
@@ -511,9 +539,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
   def test_request_cookies_wrong_domain
     uri = URI.parse 'http://host.example.com'
-    Mechanize::Cookie.parse uri, 'hello=world domain=.example.com' do |cookie|
-      @agent.cookie_jar.add uri, cookie
-    end
+    @agent.cookie_jar.parse 'hello=world domain=.example.com', uri
 
     @agent.request_cookies @req, @uri
 
@@ -671,10 +697,19 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_nil params
   end
 
+  def test_resolve_slashes
+    page = Mechanize::Page.new URI('http://example/foo/'), nil, '', 200, @mech
+    uri = '/bar/http://example/test/'
+
+    resolved = @agent.resolve uri, page
+
+    assert_equal 'http://example/bar/http://example/test/', resolved.to_s
+  end
+
   def test_response_authenticate
+    @agent.add_auth @uri, 'user', 'password'
+
     @res.instance_variable_set :@header, 'www-authenticate' => ['Basic realm=r']
-    @agent.user = 'user'
-    @agent.password = 'password'
 
     @agent.response_authenticate @res, nil, @uri, @req, {}, nil, nil
 
@@ -684,10 +719,10 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
   end
 
   def test_response_authenticate_digest
+    @agent.add_auth @uri, 'user', 'password'
+
     @res.instance_variable_set(:@header,
                                'www-authenticate' => ['Digest realm=r'])
-    @agent.user = 'user'
-    @agent.password = 'password'
 
     @agent.response_authenticate @res, nil, @uri, @req, {}, nil, nil
 
@@ -695,17 +730,19 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     realm = Mechanize::HTTP::AuthRealm.new 'Digest', base_uri, 'r'
     assert_equal [realm], @agent.authenticate_methods[base_uri][:digest]
 
-    challenge = Mechanize::HTTP::AuthChallenge.new 'Digest', 'realm' => 'r'
+    challenge = Mechanize::HTTP::AuthChallenge.new('Digest',
+                                                   { 'realm' => 'r' },
+                                                   'Digest realm=r')
+
     assert_equal challenge, @agent.digest_challenges[realm]
   end
 
   def test_response_authenticate_digest_iis
+    @agent.add_auth @uri, 'user', 'password'
+
     @res.instance_variable_set(:@header,
                                'www-authenticate' => ['Digest realm=r'],
                                'server'           => ['Microsoft-IIS'])
-    @agent.user = 'user'
-    @agent.password = 'password'
-
     @agent.response_authenticate @res, nil, @uri, @req, {}, nil, nil
 
     base_uri = @uri + '/'
@@ -714,11 +751,11 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
   end
 
   def test_response_authenticate_multiple
+    @agent.add_auth @uri, 'user', 'password'
+
     @res.instance_variable_set(:@header,
                                'www-authenticate' =>
                                  ['Basic realm=r, Digest realm=r'])
-    @agent.user = 'user'
-    @agent.password = 'password'
 
     @agent.response_authenticate @res, nil, @uri, @req, {}, nil, nil
 
@@ -729,25 +766,39 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_empty @agent.authenticate_methods[base_uri][:basic]
   end
 
+  def test_response_authenticate_no_credentials
+    @res.instance_variable_set :@header, 'www-authenticate' => ['Basic realm=r']
+
+    e = assert_raises Mechanize::UnauthorizedError do
+      @agent.response_authenticate @res, fake_page, @uri, @req, {}, nil, nil
+    end
+
+    assert_match 'no credentials', e.message
+    assert_match 'available realms: r', e.message
+  end
+
   def test_response_authenticate_no_www_authenticate
-    denied = page URI('http://example/denied'), 'text/html', '', 403
-    @agent.user = 'user'
-    @agent.password = 'password'
+    @agent.add_auth @uri, 'user', 'password'
+
+    denied_uri = URI('http://example/denied')
+
+    denied = page denied_uri, 'text/html', '', 401
 
     e = assert_raises Mechanize::UnauthorizedError do
       @agent.response_authenticate @res, denied, @uri, @req, {}, nil, nil
     end
 
-    assert_equal '403 => Net::HTTPForbidden', e.message
+    assert_equal "401 => Net::HTTPUnauthorized for #{denied_uri} -- " \
+                 "WWW-Authenticate header missing in response",
+                 e.message
   end
 
   def test_response_authenticate_ntlm
     @uri += '/ntlm'
+    @agent.add_auth @uri, 'user', 'password'
+
     @res.instance_variable_set(:@header,
                                'www-authenticate' => ['Negotiate, NTLM'])
-    @agent.user = 'user'
-    @agent.password = 'password'
-    @agent.domain = 'domain'
 
     page = @agent.response_authenticate @res, nil, @uri, @req, {}, nil, nil
 
@@ -755,8 +806,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
   end
 
   def test_response_authenticate_unknown
-    @agent.user = 'user'
-    @agent.password = 'password'
+    @agent.add_auth @uri, 'user', 'password'
+
     page = Mechanize::File.new nil, nil, nil, 401
     @res.instance_variable_set(:@header,
                                'www-authenticate' => ['Unknown realm=r'])
@@ -864,13 +915,49 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_match %r%error handling content-encoding gzip:%, e.message
     assert_match %r%Zlib%, e.message
 
-    assert_match %r%unable to gunzip response, trying raw inflate%, log.string
-    assert_match %r%unable to gunzip response:%, log.string
+    assert_match %r%unable to gunzip response: unexpected end of file%,
+                 log.string
+    assert_match %r%unable to inflate response: buffer error%,
+                 log.string
 
     assert body_io.closed?
   end
 
-  def test_response_content_encoding_gzip_corrupt_checksum
+  def test_response_content_encoding_gzip_checksum_corrupt_crc
+    log = StringIO.new
+    logger = Logger.new log
+    @agent.context.log = logger
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[gzip]
+    body_io = StringIO.new \
+      "\037\213\b\0002\002\225M\000\003+H,*\001\000\306p\017J\004\000\000\000"
+
+    body = @agent.response_content_encoding @res, body_io
+
+    assert_equal 'part', body.read
+
+    assert body_io.closed?
+
+    assert_match %r%invalid compressed data -- crc error%, log.string
+  end
+
+  def test_response_content_encoding_gzip_checksum_corrupt_length
+    log = StringIO.new
+    logger = Logger.new log
+    @agent.context.log = logger
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[gzip]
+    body_io = StringIO.new \
+      "\037\213\b\0002\002\225M\000\003+H,*\001\000\306p\017I\005\000\000\000"
+
+    @agent.response_content_encoding @res, body_io
+
+    assert body_io.closed?
+
+    assert_match %r%invalid compressed data -- length error%, log.string
+  end
+
+  def test_response_content_encoding_gzip_checksum_truncated
     log = StringIO.new
     logger = Logger.new log
     @agent.context.log = logger
@@ -879,14 +966,11 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     body_io = StringIO.new \
       "\037\213\b\0002\002\225M\000\003+H,*\001\000\306p\017I\004\000\000"
 
-    e = assert_raises Mechanize::Error do
-      @agent.response_content_encoding @res, body_io
-    end
+    @agent.response_content_encoding @res, body_io
 
-    assert_match %r%error handling content-encoding gzip:%, e.message
-    assert_match %r%Zlib%, e.message
+    assert body_io.closed?
 
-    assert_match %r%unable to gunzip response, trying raw inflate%, log.string
+    assert_match %r%unable to gunzip response: footer is not found%, log.string
   end
 
   def test_response_content_encoding_gzip_empty
@@ -916,6 +1000,18 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal Encoding::BINARY, content.encoding if have_encoding?
   end
 
+  def test_response_content_encoding_gzip_no_footer
+    @res.instance_variable_set :@header, 'content-encoding' => %w[gzip]
+    body_io = StringIO.new \
+      "\037\213\b\0002\002\225M\000\003+H,*\001\000"
+
+    body = @agent.response_content_encoding @res, body_io
+
+    assert_equal 'part', body.read
+
+    assert body_io.closed?
+  end
+
   def test_response_content_encoding_none
     @res.instance_variable_set :@header, 'content-encoding' => %w[none]
 
@@ -924,6 +1020,14 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal 'part', body.read
   end
 
+  def test_response_content_encoding_empty_string
+    @res.instance_variable_set :@header, 'content-encoding' => %w[]
+
+    body = @agent.response_content_encoding @res, StringIO.new('part')
+
+    assert_equal 'part', body.read
+  end
+
   def test_response_content_encoding_tempfile_7_bit
     body_io = tempfile 'part'
 
@@ -935,7 +1039,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     refute body_io.closed?
   ensure
     begin
-      body_io.close! unless body_io.closed?
+      body_io.close! if body_io and not body_io.closed?
     rescue IOError
       # HACK for ruby 1.8
     end
@@ -950,7 +1054,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal 'part', body.read
     assert body_io.closed?
   ensure
-    body_io.close! unless body_io.closed?
+    body_io.close! if body_io and not body_io.closed?
   end
 
   def test_response_content_encoding_unknown
@@ -984,7 +1088,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     @agent.response_cookies @res, uri, page
 
-    assert_equal ['a=b domain=.example.com'],
+    assert_equal ['a="b domain=.example.com"'],
                  @agent.cookie_jar.cookies(uri).map { |c| c.to_s }
   end
 
@@ -992,15 +1096,21 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     uri = URI.parse 'http://host.example.com'
     cookie1 = 'a=b domain=.example.com'
     cookie2 = 'c=d domain=.example.com'
+    cookies = [cookie1, cookie2]
     @res.instance_variable_set(:@header,
-                               'set-cookie' => [cookie1, cookie2],
+                               'set-cookie' => cookies,
                                'content-type' => %w[text/html])
     page = Mechanize::Page.new uri, @res, '', 200, @mech
 
     @agent.response_cookies @res, uri, page
 
-    assert_equal ['a=b domain=.example.com', 'c=d domain=.example.com'],
-                 @agent.cookie_jar.cookies(uri).map { |c| c.to_s }
+    cookies_from_jar = @agent.cookie_jar.cookies(uri)
+
+    assert_equal 2, cookies_from_jar.length
+    assert_equal [
+      'a="b domain=.example.com"',
+      'c="d domain=.example.com"',
+    ], cookies_from_jar.sort_by { |c| c.name }.map(&:to_s)
   end
 
   def test_response_cookies_meta
@@ -1019,10 +1129,28 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     @agent.response_cookies @res, uri, page
 
-    assert_equal ['a=b domain=.example.com'],
+    assert_equal ['a="b domain=.example.com"'],
                  @agent.cookie_jar.cookies(uri).map { |c| c.to_s }
   end
 
+  def test_response_cookies_meta_bogus
+    uri = URI.parse 'http://host.example.com'
+
+    body = <<-BODY
+<head>
+  <meta http-equiv="Set-Cookie">
+</head>"
+    BODY
+
+    @res.instance_variable_set(:@header,
+                               'content-type' => %w[text/html])
+    page = Mechanize::Page.new uri, @res, body, 200, @mech
+
+    @agent.response_cookies @res, uri, page
+
+    assert_empty @agent.cookie_jar.cookies(uri)
+  end
+
   def test_response_follow_meta_refresh
     uri = URI.parse 'http://example/#id+1'
 
@@ -1158,6 +1286,21 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal Encoding::BINARY, body.encoding if body.respond_to? :encoding
   end
 
+  def test_response_read_chunked_no_trailer
+    @res['Transfer-Encoding'] = 'chunked'
+    def @res.content_length() end
+    def @res.read_body
+      yield 'a' * 10
+      raise EOFError
+    end
+
+    e = assert_raises Mechanize::ChunkedTerminationError do
+      @agent.response_read @res, @req, @uri
+    end
+
+    assert_equal 'aaaaaaaaaa', e.body_io.read
+  end
+
   def test_response_read_content_length_head
     req = Net::HTTP::Head.new '/'
 
@@ -1173,12 +1316,12 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     def @res.content_length() 5 end
     def @res.read_body() yield 'part' end
 
-    e = assert_raises EOFError do
+    e = assert_raises Mechanize::ResponseReadError do
       @agent.response_read @res, @req, @uri
     end
 
-    assert_equal 'Content-Length (5) does not match response body length (4)',
-                 e.message
+    assert_equal 'Content-Length (5) does not match response body length (4)' \
+      ' (Mechanize::ResponseReadError)', e.message
   end
 
   def test_response_read_content_length_redirect
@@ -1223,8 +1366,13 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
       expected = "π\n"
       expected.force_encoding Encoding::BINARY if expected.respond_to? :encoding
 
+      # Ruby 1.8.7 doesn't let us set the write mode of the tempfile to binary,
+      # so we should expect an inserted carriage return on some platforms
+      expected_with_carriage_return = "π\r\n"
+      expected_with_carriage_return.force_encoding Encoding::BINARY if expected_with_carriage_return.respond_to? :encoding
+
       body = io.read
-      assert_equal expected, body
+      assert_match(/^(#{expected}|#{expected_with_carriage_return})$/m, body)
       assert_equal Encoding::BINARY, body.encoding if body.respond_to? :encoding
     end
   end
@@ -1285,20 +1433,41 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     page = fake_page
     page = @agent.response_redirect({ 'Location' => '/index.html' }, :get,
-                                    page, 0, referer)
+                                    page, 0, {}, referer)
 
     assert_equal URI('http://fake.example/index.html'), page.uri
 
     assert_equal 'http://example/referer', requests.first['Referer']
   end
 
+  def test_response_redirect_header
+    @agent.redirect_ok = true
+    referer = page 'http://example/referer'
+
+    headers = {
+      'Range' => 'bytes=0-9999',
+      'Content-Type' => 'application/x-www-form-urlencoded',
+      'Content-Length' => '9999',
+    }
+
+    page = fake_page
+    page = @agent.response_redirect({ 'Location' => '/http_headers' }, :get,
+                                    page, 0, headers, referer)
+
+    assert_equal URI('http://fake.example/http_headers'), page.uri
+
+    assert_match 'range|bytes=0-9999', page.body
+    refute_match 'content-type|application/x-www-form-urlencoded', page.body
+    refute_match 'content-length|9999', page.body
+  end
+
   def test_response_redirect_malformed
     @agent.redirect_ok = true
     referer = page 'http://example/referer'
 
     page = fake_page
     page = @agent.response_redirect({ 'Location' => '/index.html?q=あ' }, :get,
-                                    page, 0, referer)
+                                    page, 0, {}, referer)
 
     assert_equal URI('http://fake.example/index.html?q=%E3%81%82'), page.uri
 
@@ -1311,7 +1480,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     assert_raises Mechanize::RedirectLimitReachedError do
       @agent.response_redirect({ 'Location' => '/index.html' }, :get,
-                               fake_page, @agent.redirection_limit, referer)
+                               fake_page, @agent.redirection_limit, {}, referer)
     end
   end
 
@@ -1320,7 +1489,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     page = fake_page
     page = @agent.response_redirect({ 'Location' => '/other' }, :get, page, 0,
-                                    page)
+                                    {}, page)
 
     assert_equal URI('http://fake.example'), page.uri
   end
@@ -1332,7 +1501,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     response.instance_variable_set :@header, { 'location' => %w[/index.html] }
 
     page = fake_page
-    page = @agent.response_redirect response, :get, page, 0, page
+    page = @agent.response_redirect response, :get, page, 0, {}, page
 
     assert_equal URI('http://fake.example/index.html'), page.uri
   end
@@ -1344,7 +1513,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     response.instance_variable_set :@header, { 'location' => %w[/index.html] }
 
     page = fake_page
-    page = @agent.response_redirect response, :get, page, 0, page
+    page = @agent.response_redirect response, :get, page, 0, {}, page
 
     assert_equal URI('http://fake.example/'), page.uri
   end
