@@ -8,7 +8,7 @@ class TestMechanizeHttpAuthChallenge < Mechanize::TestCase
     @uri = URI 'http://example/'
     @AR = Mechanize::HTTP::AuthRealm
     @AC = Mechanize::HTTP::AuthChallenge
-    @challenge = @AC.new 'Digest', 'realm' => 'r'
+    @challenge = @AC.new 'Digest', { 'realm' => 'r' }, 'Digest realm=r'
   end
 
   def test_realm_basic
@@ -35,5 +35,15 @@ class TestMechanizeHttpAuthChallenge < Mechanize::TestCase
     assert_equal 'unknown HTTP authentication scheme Unknown', e.message
   end
 
+  def test_realm_name
+    assert_equal 'r', @challenge.realm_name
+  end
+
+  def test_realm_name_ntlm
+    challenge = @AC.new 'Negotiate, NTLM'
+
+    assert_nil challenge.realm_name
+  end
+
 end
 
